#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@author: P.Cizek
"""
import sys
import numpy as np

import math

import collections
import heapq

import map as mp

import matplotlib.pyplot as plt
from scipy import spatial

import imp
try:
	imp.find_module('ompl')
	found = True
except ImportError:
	found = False
if found:
	from ompl import base as ob
	from ompl import geometric as og


class Planner:
	def __init__(self):
		self.used_planner = self.a_star_search
	
	#########################################
	# OrientedGraph class
	#########################################
	class OrientedGraph:
		def __init__(self, vertices, edges):
			self.vertices = vertices
			self.edges = edges
		
		def neighbors(self, id):
			#returns coordinates of graph neighbors in the oriented graph
			d = [item for item in self.edges if item[0] == id]
			if d != None:
				res = [x[1] for x in d]
			else:
				res = None
			return res
		
		def cost(self, current, next):
			#squared euclidean distance between two points
			(x1, y1) = current
			(x2, y2) = next
			return math.sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2))
	
	#########################################
	# Plotting functions
	#########################################
	def plot_points(self, points, specs):
		x_val = [x[0] for x in points]
		y_val = [x[1] for x in points]
		plt.plot(x_val, y_val, specs)
		plt.draw()
	
	def plot_edges(self, edges, specs):
		for tup in edges:
			x_val = [x[0] for x in tup]
			y_val = [x[1] for x in tup] 
			plt.plot(x_val, y_val, specs)
		plt.draw()
		
	#########################################
	# A_STAR
	#########################################
	
	class PriorityQueue:
		def __init__(self):
			self.elements = []
	
		def empty(self):
			return len(self.elements) == 0
	
		def put(self, item, priority):
			heapq.heappush(self.elements, (priority, item))
	
		def get(self):
			return heapq.heappop(self.elements)[1]
	
	def heuristic(self, a, b):
			(x1, y1) = a
			(x2, y2) = b
			return math.sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2))
	
	def a_star_search(self, graph, start, goal):
		open_set = self.PriorityQueue()
		open_set.put(start, 0)
		came_from = {}
		cost_so_far = {}
		came_from[start] = None
		cost_so_far[start] = 0
		
		while not open_set.empty():
			current = open_set.get()
			if current == goal:
				break
		
			for next in graph.neighbors(current):
				new_cost = cost_so_far[current] + graph.cost(current, next)
				if next not in cost_so_far or new_cost < cost_so_far[next]:
					cost_so_far[next] = new_cost
					priority = new_cost + self.heuristic(goal, next)
					open_set.put(next, priority)
					came_from[next] = current
	
		return came_from, cost_so_far
	
	#########################################
	# backtracking function
	#########################################
	
	def reconstruct_path(self, came_from, start, goal):
		current = goal
		path = [current]
		while current != start:
			current = came_from[current]
			path.append(current)
		path.append(start) # optional
		path.reverse() # optional
		return path
	
	#########################################
	# Planning function
	#########################################
	
	def plan(self, map, start, goal):
		path = []
		num_sampled_points = 400
		
		#randomly sample points
		sampled_points = zip(np.random.randint(map.get_width(), size=num_sampled_points), 
							 np.random.randint(map.get_height(), size=num_sampled_points))
		sampled_points.append(start)
		sampled_points.append(goal)
		
		#plot the points
		mp.plot_map(map.get_grid())
		self.plot_points(sampled_points, 'or')
		plt.show()
		
		"""
		path planning on the probabilistic roadmap
		"""
		return path
	
	#########################################
	# Execution function
	# -drive the robot through the maze
	#########################################
	
	def execute(self, map, start, goal):
		path = self.plan(map, start, goal)
		"""
		robot control
		"""
		return True
	
	#########################################
	# Planning function ompl
	#########################################
	
	def plan_ompl(self, map, start, goal):
		path = []
		"""
		path planning using the open motion planning library
		"""
		return path
	
	#########################################
	# Execution function
	# -drive the robot through the maze
	#########################################
	
	def execute_ompl(self, map, start, goal):
		path = self.plan_ompl(map, start, goal)
		"""
		robot control
		"""
		return True